# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

WARPINIT_METADATA = Metadata(
    id="280e00eee2bd09c4c86c25af7d4cac9f4251a8af.boutiques",
    name="warpinit",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_WarpinitConfigParamsDictNoTag = typing.TypedDict('_WarpinitConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
WarpinitConfigParamsDictTagged = typing.TypedDict('WarpinitConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
WarpinitConfigParamsDict = _WarpinitConfigParamsDictNoTag | WarpinitConfigParamsDictTagged


_WarpinitParamsDictNoTag = typing.TypedDict('_WarpinitParamsDictNoTag', {
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[WarpinitConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "template": InputPathType,
    "warp": str,
})
WarpinitParamsDictTagged = typing.TypedDict('WarpinitParamsDictTagged', {
    "@type": typing.Literal["mrtrix/warpinit"],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[WarpinitConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "template": InputPathType,
    "warp": str,
})
WarpinitParamsDict = _WarpinitParamsDictNoTag | WarpinitParamsDictTagged


def warpinit_config(
    key: str,
    value: str,
) -> WarpinitConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def warpinit_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `WarpinitConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def warpinit_config_cargs(
    params: WarpinitConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class WarpinitOutputs(typing.NamedTuple):
    """
    Output object returned when calling `WarpinitParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    warp: OutputPathType
    """the output warp image."""


def warpinit_params(
    template: InputPathType,
    warp: str,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[WarpinitConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> WarpinitParamsDictTagged:
    """
    Build parameters.
    
    Args:
        template: the input template image.
        warp: the output warp image.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/warpinit",
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "template": template,
        "warp": warp,
    }
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def warpinit_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `WarpinitParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[WarpinitConfigParamsDict] | None`')
        for e in params["config"]:
            warpinit_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("template", None) is None:
        raise StyxValidationError("`template` must not be None")
    if not isinstance(params["template"], (pathlib.Path, str)):
        raise StyxValidationError(f'`template` has the wrong type: Received `{type(params.get("template", None))}` expected `InputPathType`')
    if params.get("warp", None) is None:
        raise StyxValidationError("`warp` must not be None")
    if not isinstance(params["warp"], str):
        raise StyxValidationError(f'`warp` has the wrong type: Received `{type(params.get("warp", None))}` expected `str`')


def warpinit_cargs(
    params: WarpinitParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("warpinit")
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [warpinit_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("template", None)))
    cargs.append(params.get("warp", None))
    return cargs


def warpinit_outputs(
    params: WarpinitParamsDict,
    execution: Execution,
) -> WarpinitOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = WarpinitOutputs(
        root=execution.output_file("."),
        warp=execution.output_file(params.get("warp", None)),
    )
    return ret


def warpinit_execute(
    params: WarpinitParamsDict,
    runner: Runner | None = None,
) -> WarpinitOutputs:
    """
    warpinit
    
    Create an initial warp image, representing an identity transformation.
    
    This is useful to obtain the warp fields from other normalisation
    applications, by applying the transformation of interest to the warp field
    generated by this program.
    
    The image generated is a 4D image with the same spatial characteristics as
    the input template image. It contains 3 volumes, with each voxel containing
    its own x,y,z coordinates.
    
    Note that this command can be used to create 3 separate X,Y,Z images
    directly (which may be useful to create images suitable for use in the
    registration program) using the following syntax:
    
    $ warpinit template.mif warp-'[]'.nii
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `WarpinitOutputs`).
    """
    warpinit_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(WARPINIT_METADATA)
    params = execution.params(params)
    cargs = warpinit_cargs(params, execution)
    ret = warpinit_outputs(params, execution)
    execution.run(cargs)
    return ret


def warpinit(
    template: InputPathType,
    warp: str,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[WarpinitConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> WarpinitOutputs:
    """
    warpinit
    
    Create an initial warp image, representing an identity transformation.
    
    This is useful to obtain the warp fields from other normalisation
    applications, by applying the transformation of interest to the warp field
    generated by this program.
    
    The image generated is a 4D image with the same spatial characteristics as
    the input template image. It contains 3 volumes, with each voxel containing
    its own x,y,z coordinates.
    
    Note that this command can be used to create 3 separate X,Y,Z images
    directly (which may be useful to create images suitable for use in the
    registration program) using the following syntax:
    
    $ warpinit template.mif warp-'[]'.nii
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        template: the input template image.
        warp: the output warp image.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `WarpinitOutputs`).
    """
    params = warpinit_params(
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        template=template,
        warp=warp,
    )
    return warpinit_execute(params, runner)


__all__ = [
    "WARPINIT_METADATA",
    "WarpinitConfigParamsDict",
    "WarpinitConfigParamsDictTagged",
    "WarpinitOutputs",
    "WarpinitParamsDict",
    "WarpinitParamsDictTagged",
    "warpinit",
    "warpinit_config",
    "warpinit_execute",
    "warpinit_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CONNECTOMESTATS_METADATA = Metadata(
    id="607b512a43f94c7e5deeb72046391536e66cc167.boutiques",
    name="connectomestats",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_ConnectomestatsColumnParamsDictNoTag = typing.TypedDict('_ConnectomestatsColumnParamsDictNoTag', {
    "path": InputPathType,
})
ConnectomestatsColumnParamsDictTagged = typing.TypedDict('ConnectomestatsColumnParamsDictTagged', {
    "@type": typing.Literal["column"],
    "path": InputPathType,
})
ConnectomestatsColumnParamsDict = _ConnectomestatsColumnParamsDictNoTag | ConnectomestatsColumnParamsDictTagged


_ConnectomestatsConfigParamsDictNoTag = typing.TypedDict('_ConnectomestatsConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
ConnectomestatsConfigParamsDictTagged = typing.TypedDict('ConnectomestatsConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
ConnectomestatsConfigParamsDict = _ConnectomestatsConfigParamsDictNoTag | ConnectomestatsConfigParamsDictTagged


_ConnectomestatsParamsDictNoTag = typing.TypedDict('_ConnectomestatsParamsDictNoTag', {
    "notest": bool,
    "errors": typing.NotRequired[str | None],
    "exchange_within": typing.NotRequired[InputPathType | None],
    "exchange_whole": typing.NotRequired[InputPathType | None],
    "strong": bool,
    "nshuffles": typing.NotRequired[int | None],
    "permutations": typing.NotRequired[InputPathType | None],
    "nonstationarity": bool,
    "skew_nonstationarity": typing.NotRequired[float | None],
    "nshuffles_nonstationarity": typing.NotRequired[int | None],
    "permutations_nonstationarity": typing.NotRequired[InputPathType | None],
    "tfce_dh": typing.NotRequired[float | None],
    "tfce_e": typing.NotRequired[float | None],
    "tfce_h": typing.NotRequired[float | None],
    "variance": typing.NotRequired[InputPathType | None],
    "ftests": typing.NotRequired[InputPathType | None],
    "fonly": bool,
    "column": typing.NotRequired[list[ConnectomestatsColumnParamsDict] | None],
    "threshold": typing.NotRequired[float | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[ConnectomestatsConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "input": InputPathType,
    "algorithm": str,
    "design": InputPathType,
    "contrast": InputPathType,
    "output": str,
})
ConnectomestatsParamsDictTagged = typing.TypedDict('ConnectomestatsParamsDictTagged', {
    "@type": typing.Literal["mrtrix/connectomestats"],
    "notest": bool,
    "errors": typing.NotRequired[str | None],
    "exchange_within": typing.NotRequired[InputPathType | None],
    "exchange_whole": typing.NotRequired[InputPathType | None],
    "strong": bool,
    "nshuffles": typing.NotRequired[int | None],
    "permutations": typing.NotRequired[InputPathType | None],
    "nonstationarity": bool,
    "skew_nonstationarity": typing.NotRequired[float | None],
    "nshuffles_nonstationarity": typing.NotRequired[int | None],
    "permutations_nonstationarity": typing.NotRequired[InputPathType | None],
    "tfce_dh": typing.NotRequired[float | None],
    "tfce_e": typing.NotRequired[float | None],
    "tfce_h": typing.NotRequired[float | None],
    "variance": typing.NotRequired[InputPathType | None],
    "ftests": typing.NotRequired[InputPathType | None],
    "fonly": bool,
    "column": typing.NotRequired[list[ConnectomestatsColumnParamsDict] | None],
    "threshold": typing.NotRequired[float | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[ConnectomestatsConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "input": InputPathType,
    "algorithm": str,
    "design": InputPathType,
    "contrast": InputPathType,
    "output": str,
})
ConnectomestatsParamsDict = _ConnectomestatsParamsDictNoTag | ConnectomestatsParamsDictTagged


def connectomestats_column(
    path: InputPathType,
) -> ConnectomestatsColumnParamsDictTagged:
    """
    Build parameters.
    
    Args:
        path: add a column to the design matrix corresponding to subject\
            edge-wise values (note that the contrast matrix must include an\
            additional column for each use of this option); the text file provided\
            via this option should contain a file name for each subject.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "column",
        "path": path,
    }
    return params


def connectomestats_column_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `ConnectomestatsColumnParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("path", None) is None:
        raise StyxValidationError("`path` must not be None")
    if not isinstance(params["path"], (pathlib.Path, str)):
        raise StyxValidationError(f'`path` has the wrong type: Received `{type(params.get("path", None))}` expected `InputPathType`')


def connectomestats_column_cargs(
    params: ConnectomestatsColumnParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-column")
    cargs.append(execution.input_file(params.get("path", None)))
    return cargs


def connectomestats_config(
    key: str,
    value: str,
) -> ConnectomestatsConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def connectomestats_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `ConnectomestatsConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def connectomestats_config_cargs(
    params: ConnectomestatsConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class ConnectomestatsOutputs(typing.NamedTuple):
    """
    Output object returned when calling `ConnectomestatsParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def connectomestats_params(
    input_: InputPathType,
    algorithm: str,
    design: InputPathType,
    contrast: InputPathType,
    output: str,
    notest: bool = False,
    errors: str | None = None,
    exchange_within: InputPathType | None = None,
    exchange_whole: InputPathType | None = None,
    strong: bool = False,
    nshuffles: int | None = None,
    permutations: InputPathType | None = None,
    nonstationarity: bool = False,
    skew_nonstationarity: float | None = None,
    nshuffles_nonstationarity: int | None = None,
    permutations_nonstationarity: InputPathType | None = None,
    tfce_dh: float | None = None,
    tfce_e: float | None = None,
    tfce_h: float | None = None,
    variance: InputPathType | None = None,
    ftests: InputPathType | None = None,
    fonly: bool = False,
    column: list[ConnectomestatsColumnParamsDict] | None = None,
    threshold: float | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[ConnectomestatsConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> ConnectomestatsParamsDictTagged:
    """
    Build parameters.
    
    Args:
        input_: a text file listing the file names of the input connectomes.
        algorithm: the algorithm to use in network-based\
            clustering/enhancement. Options are: nbs, tfnbs, none.
        design: the design matrix.
        contrast: the contrast matrix.
        output: the filename prefix for all output.
        notest: don't perform statistical inference; only output population\
            statistics (effect size, stdev etc).
        errors: specify nature of errors for shuffling; options are:\
            ee,ise,both (default: ee).
        exchange_within: specify blocks of observations within each of which\
            data may undergo restricted exchange.
        exchange_whole: specify blocks of observations that may be exchanged\
            with one another (for independent and symmetric errors, sign-flipping\
            will occur block-wise).
        strong: use strong familywise error control across multiple hypotheses.
        nshuffles: the number of shuffles (default: 5000).
        permutations: manually define the permutations (relabelling). The input\
            should be a text file defining a m x n matrix, where each relabelling\
            is defined as a column vector of size m, and the number of columns, n,\
            defines the number of permutations. Can be generated with the\
            palm_quickperms function in PALM\
            (http://fsl.fmrib.ox.ac.uk/fsl/fslwiki/PALM). Overrides the -nshuffles\
            option.
        nonstationarity: perform non-stationarity correction.
        skew_nonstationarity: specify the skew parameter for empirical\
            statistic calculation (default for this command is 1).
        nshuffles_nonstationarity: the number of shuffles to use when\
            precomputing the empirical statistic image for non-stationarity\
            correction (default: 5000).
        permutations_nonstationarity: manually define the permutations\
            (relabelling) for computing the emprical statistics for\
            non-stationarity correction. The input should be a text file defining a\
            m x n matrix, where each relabelling is defined as a column vector of\
            size m, and the number of columns, n, defines the number of\
            permutations. Can be generated with the palm_quickperms function in\
            PALM (http://fsl.fmrib.ox.ac.uk/fsl/fslwiki/PALM) Overrides the\
            -nshuffles_nonstationarity option.
        tfce_dh: the height increment used in the tfce integration (default:\
            0.1).
        tfce_e: tfce extent exponent (default: 0.4).
        tfce_h: tfce height exponent (default: 3).
        variance: define variance groups for the G-statistic; measurements for\
            which the expected variance is equivalent should contain the same index.
        ftests: perform F-tests; input text file should contain, for each\
            F-test, a row containing ones and zeros, where ones indicate the rows\
            of the contrast matrix to be included in the F-test.
        fonly: only assess F-tests; do not perform statistical inference on\
            entries in the contrast matrix.
        column: add a column to the design matrix corresponding to subject\
            edge-wise values (note that the contrast matrix must include an\
            additional column for each use of this option); the text file provided\
            via this option should contain a file name for each subject.
        threshold: the t-statistic value to use in threshold-based clustering\
            algorithms.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/connectomestats",
        "notest": notest,
        "strong": strong,
        "nonstationarity": nonstationarity,
        "fonly": fonly,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "input": input_,
        "algorithm": algorithm,
        "design": design,
        "contrast": contrast,
        "output": output,
    }
    if errors is not None:
        params["errors"] = errors
    if exchange_within is not None:
        params["exchange_within"] = exchange_within
    if exchange_whole is not None:
        params["exchange_whole"] = exchange_whole
    if nshuffles is not None:
        params["nshuffles"] = nshuffles
    if permutations is not None:
        params["permutations"] = permutations
    if skew_nonstationarity is not None:
        params["skew_nonstationarity"] = skew_nonstationarity
    if nshuffles_nonstationarity is not None:
        params["nshuffles_nonstationarity"] = nshuffles_nonstationarity
    if permutations_nonstationarity is not None:
        params["permutations_nonstationarity"] = permutations_nonstationarity
    if tfce_dh is not None:
        params["tfce_dh"] = tfce_dh
    if tfce_e is not None:
        params["tfce_e"] = tfce_e
    if tfce_h is not None:
        params["tfce_h"] = tfce_h
    if variance is not None:
        params["variance"] = variance
    if ftests is not None:
        params["ftests"] = ftests
    if column is not None:
        params["column"] = column
    if threshold is not None:
        params["threshold"] = threshold
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def connectomestats_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `ConnectomestatsParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("notest", False) is None:
        raise StyxValidationError("`notest` must not be None")
    if not isinstance(params["notest"], bool):
        raise StyxValidationError(f'`notest` has the wrong type: Received `{type(params.get("notest", False))}` expected `bool`')
    if params.get("errors", None) is not None:
        if not isinstance(params["errors"], str):
            raise StyxValidationError(f'`errors` has the wrong type: Received `{type(params.get("errors", None))}` expected `str | None`')
    if params.get("exchange_within", None) is not None:
        if not isinstance(params["exchange_within"], (pathlib.Path, str)):
            raise StyxValidationError(f'`exchange_within` has the wrong type: Received `{type(params.get("exchange_within", None))}` expected `InputPathType | None`')
    if params.get("exchange_whole", None) is not None:
        if not isinstance(params["exchange_whole"], (pathlib.Path, str)):
            raise StyxValidationError(f'`exchange_whole` has the wrong type: Received `{type(params.get("exchange_whole", None))}` expected `InputPathType | None`')
    if params.get("strong", False) is None:
        raise StyxValidationError("`strong` must not be None")
    if not isinstance(params["strong"], bool):
        raise StyxValidationError(f'`strong` has the wrong type: Received `{type(params.get("strong", False))}` expected `bool`')
    if params.get("nshuffles", None) is not None:
        if not isinstance(params["nshuffles"], int):
            raise StyxValidationError(f'`nshuffles` has the wrong type: Received `{type(params.get("nshuffles", None))}` expected `int | None`')
    if params.get("permutations", None) is not None:
        if not isinstance(params["permutations"], (pathlib.Path, str)):
            raise StyxValidationError(f'`permutations` has the wrong type: Received `{type(params.get("permutations", None))}` expected `InputPathType | None`')
    if params.get("nonstationarity", False) is None:
        raise StyxValidationError("`nonstationarity` must not be None")
    if not isinstance(params["nonstationarity"], bool):
        raise StyxValidationError(f'`nonstationarity` has the wrong type: Received `{type(params.get("nonstationarity", False))}` expected `bool`')
    if params.get("skew_nonstationarity", None) is not None:
        if not isinstance(params["skew_nonstationarity"], (float, int)):
            raise StyxValidationError(f'`skew_nonstationarity` has the wrong type: Received `{type(params.get("skew_nonstationarity", None))}` expected `float | None`')
    if params.get("nshuffles_nonstationarity", None) is not None:
        if not isinstance(params["nshuffles_nonstationarity"], int):
            raise StyxValidationError(f'`nshuffles_nonstationarity` has the wrong type: Received `{type(params.get("nshuffles_nonstationarity", None))}` expected `int | None`')
    if params.get("permutations_nonstationarity", None) is not None:
        if not isinstance(params["permutations_nonstationarity"], (pathlib.Path, str)):
            raise StyxValidationError(f'`permutations_nonstationarity` has the wrong type: Received `{type(params.get("permutations_nonstationarity", None))}` expected `InputPathType | None`')
    if params.get("tfce_dh", None) is not None:
        if not isinstance(params["tfce_dh"], (float, int)):
            raise StyxValidationError(f'`tfce_dh` has the wrong type: Received `{type(params.get("tfce_dh", None))}` expected `float | None`')
    if params.get("tfce_e", None) is not None:
        if not isinstance(params["tfce_e"], (float, int)):
            raise StyxValidationError(f'`tfce_e` has the wrong type: Received `{type(params.get("tfce_e", None))}` expected `float | None`')
    if params.get("tfce_h", None) is not None:
        if not isinstance(params["tfce_h"], (float, int)):
            raise StyxValidationError(f'`tfce_h` has the wrong type: Received `{type(params.get("tfce_h", None))}` expected `float | None`')
    if params.get("variance", None) is not None:
        if not isinstance(params["variance"], (pathlib.Path, str)):
            raise StyxValidationError(f'`variance` has the wrong type: Received `{type(params.get("variance", None))}` expected `InputPathType | None`')
    if params.get("ftests", None) is not None:
        if not isinstance(params["ftests"], (pathlib.Path, str)):
            raise StyxValidationError(f'`ftests` has the wrong type: Received `{type(params.get("ftests", None))}` expected `InputPathType | None`')
    if params.get("fonly", False) is None:
        raise StyxValidationError("`fonly` must not be None")
    if not isinstance(params["fonly"], bool):
        raise StyxValidationError(f'`fonly` has the wrong type: Received `{type(params.get("fonly", False))}` expected `bool`')
    if params.get("column", None) is not None:
        if not isinstance(params["column"], list):
            raise StyxValidationError(f'`column` has the wrong type: Received `{type(params.get("column", None))}` expected `list[ConnectomestatsColumnParamsDict] | None`')
        for e in params["column"]:
            connectomestats_column_validate(e)
    if params.get("threshold", None) is not None:
        if not isinstance(params["threshold"], (float, int)):
            raise StyxValidationError(f'`threshold` has the wrong type: Received `{type(params.get("threshold", None))}` expected `float | None`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[ConnectomestatsConfigParamsDict] | None`')
        for e in params["config"]:
            connectomestats_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("input", None) is None:
        raise StyxValidationError("`input` must not be None")
    if not isinstance(params["input"], (pathlib.Path, str)):
        raise StyxValidationError(f'`input` has the wrong type: Received `{type(params.get("input", None))}` expected `InputPathType`')
    if params.get("algorithm", None) is None:
        raise StyxValidationError("`algorithm` must not be None")
    if not isinstance(params["algorithm"], str):
        raise StyxValidationError(f'`algorithm` has the wrong type: Received `{type(params.get("algorithm", None))}` expected `str`')
    if params.get("design", None) is None:
        raise StyxValidationError("`design` must not be None")
    if not isinstance(params["design"], (pathlib.Path, str)):
        raise StyxValidationError(f'`design` has the wrong type: Received `{type(params.get("design", None))}` expected `InputPathType`')
    if params.get("contrast", None) is None:
        raise StyxValidationError("`contrast` must not be None")
    if not isinstance(params["contrast"], (pathlib.Path, str)):
        raise StyxValidationError(f'`contrast` has the wrong type: Received `{type(params.get("contrast", None))}` expected `InputPathType`')
    if params.get("output", None) is None:
        raise StyxValidationError("`output` must not be None")
    if not isinstance(params["output"], str):
        raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str`')


def connectomestats_cargs(
    params: ConnectomestatsParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("connectomestats")
    if params.get("notest", False):
        cargs.append("-notest")
    if params.get("errors", None) is not None:
        cargs.extend([
            "-errors",
            params.get("errors", None)
        ])
    if params.get("exchange_within", None) is not None:
        cargs.extend([
            "-exchange_within",
            execution.input_file(params.get("exchange_within", None))
        ])
    if params.get("exchange_whole", None) is not None:
        cargs.extend([
            "-exchange_whole",
            execution.input_file(params.get("exchange_whole", None))
        ])
    if params.get("strong", False):
        cargs.append("-strong")
    if params.get("nshuffles", None) is not None:
        cargs.extend([
            "-nshuffles",
            str(params.get("nshuffles", None))
        ])
    if params.get("permutations", None) is not None:
        cargs.extend([
            "-permutations",
            execution.input_file(params.get("permutations", None))
        ])
    if params.get("nonstationarity", False):
        cargs.append("-nonstationarity")
    if params.get("skew_nonstationarity", None) is not None:
        cargs.extend([
            "-skew_nonstationarity",
            str(params.get("skew_nonstationarity", None))
        ])
    if params.get("nshuffles_nonstationarity", None) is not None:
        cargs.extend([
            "-nshuffles_nonstationarity",
            str(params.get("nshuffles_nonstationarity", None))
        ])
    if params.get("permutations_nonstationarity", None) is not None:
        cargs.extend([
            "-permutations_nonstationarity",
            execution.input_file(params.get("permutations_nonstationarity", None))
        ])
    if params.get("tfce_dh", None) is not None:
        cargs.extend([
            "-tfce_dh",
            str(params.get("tfce_dh", None))
        ])
    if params.get("tfce_e", None) is not None:
        cargs.extend([
            "-tfce_e",
            str(params.get("tfce_e", None))
        ])
    if params.get("tfce_h", None) is not None:
        cargs.extend([
            "-tfce_h",
            str(params.get("tfce_h", None))
        ])
    if params.get("variance", None) is not None:
        cargs.extend([
            "-variance",
            execution.input_file(params.get("variance", None))
        ])
    if params.get("ftests", None) is not None:
        cargs.extend([
            "-ftests",
            execution.input_file(params.get("ftests", None))
        ])
    if params.get("fonly", False):
        cargs.append("-fonly")
    if params.get("column", None) is not None:
        cargs.extend([a for c in [connectomestats_column_cargs(s, execution) for s in params.get("column", None)] for a in c])
    if params.get("threshold", None) is not None:
        cargs.extend([
            "-threshold",
            str(params.get("threshold", None))
        ])
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [connectomestats_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("input", None)))
    cargs.append(params.get("algorithm", None))
    cargs.append(execution.input_file(params.get("design", None)))
    cargs.append(execution.input_file(params.get("contrast", None)))
    cargs.append(params.get("output", None))
    return cargs


def connectomestats_outputs(
    params: ConnectomestatsParamsDict,
    execution: Execution,
) -> ConnectomestatsOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = ConnectomestatsOutputs(
        root=execution.output_file("."),
    )
    return ret


def connectomestats_execute(
    params: ConnectomestatsParamsDict,
    runner: Runner | None = None,
) -> ConnectomestatsOutputs:
    """
    connectomestats
    
    Connectome group-wise statistics at the edge level using non-parametric
    permutation testing.
    
    For the TFNBS algorithm, default parameters for statistical enhancement have
    been set based on the work in:
    Vinokur, L.; Zalesky, A.; Raffelt, D.; Smith, R.E. & Connelly, A. A Novel
    Threshold-Free Network-Based Statistics Method: Demonstration using
    Simulated Pathology. OHBM, 2015, 4144;
    and:
    Vinokur, L.; Zalesky, A.; Raffelt, D.; Smith, R.E. & Connelly, A. A novel
    threshold-free network-based statistical method: Demonstration and parameter
    optimisation using in vivo simulated pathology. In Proc ISMRM, 2015, 2846.
    Note however that not only was the optimisation of these parameters not very
    precise, but the outcomes of statistical inference (for both this algorithm
    and the NBS method) can vary markedly for even small changes to enhancement
    parameters. Therefore the specificity of results obtained using either of
    these methods should be interpreted with caution.
    
    In some software packages, a column of ones is automatically added to the
    GLM design matrix; the purpose of this column is to estimate the "global
    intercept", which is the predicted value of the observed variable if all
    explanatory variables were to be zero. However there are rare situations
    where including such a column would not be appropriate for a particular
    experimental design. Hence, in MRtrix3 statistical inference commands, it is
    up to the user to determine whether or not this column of ones should be
    included in their design matrix, and add it explicitly if necessary. The
    contrast matrix must also reflect the presence of this additional column.
    
    References:
    
    * If using the NBS algorithm:
    Zalesky, A.; Fornito, A. & Bullmore, E. T. Network-based statistic:
    Identifying differences in brain networks.
    NeuroImage, 2010, 53, 1197-1207
    
    * If using the TFNBS algorithm:
    Baggio, H.C.; Abos, A.; Segura, B.; Campabadal, A.; Garcia-Diaz, A.; Uribe,
    C.; Compta, Y.; Marti, M.J.; Valldeoriola, F.; Junque, C. Statistical
    inference in brain graphs using threshold-free network-based statistics.HBM,
    2018, 39, 2289-2302
    
    * If using the -nonstationary option:
    Salimi-Khorshidi, G.; Smith, S.M. & Nichols, T.E. Adjusting the effect of
    nonstationarity in cluster-based and TFCE inference.
    Neuroimage, 2011, 54(3), 2006-19.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ConnectomestatsOutputs`).
    """
    connectomestats_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CONNECTOMESTATS_METADATA)
    params = execution.params(params)
    cargs = connectomestats_cargs(params, execution)
    ret = connectomestats_outputs(params, execution)
    execution.run(cargs)
    return ret


def connectomestats(
    input_: InputPathType,
    algorithm: str,
    design: InputPathType,
    contrast: InputPathType,
    output: str,
    notest: bool = False,
    errors: str | None = None,
    exchange_within: InputPathType | None = None,
    exchange_whole: InputPathType | None = None,
    strong: bool = False,
    nshuffles: int | None = None,
    permutations: InputPathType | None = None,
    nonstationarity: bool = False,
    skew_nonstationarity: float | None = None,
    nshuffles_nonstationarity: int | None = None,
    permutations_nonstationarity: InputPathType | None = None,
    tfce_dh: float | None = None,
    tfce_e: float | None = None,
    tfce_h: float | None = None,
    variance: InputPathType | None = None,
    ftests: InputPathType | None = None,
    fonly: bool = False,
    column: list[ConnectomestatsColumnParamsDict] | None = None,
    threshold: float | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[ConnectomestatsConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> ConnectomestatsOutputs:
    """
    connectomestats
    
    Connectome group-wise statistics at the edge level using non-parametric
    permutation testing.
    
    For the TFNBS algorithm, default parameters for statistical enhancement have
    been set based on the work in:
    Vinokur, L.; Zalesky, A.; Raffelt, D.; Smith, R.E. & Connelly, A. A Novel
    Threshold-Free Network-Based Statistics Method: Demonstration using
    Simulated Pathology. OHBM, 2015, 4144;
    and:
    Vinokur, L.; Zalesky, A.; Raffelt, D.; Smith, R.E. & Connelly, A. A novel
    threshold-free network-based statistical method: Demonstration and parameter
    optimisation using in vivo simulated pathology. In Proc ISMRM, 2015, 2846.
    Note however that not only was the optimisation of these parameters not very
    precise, but the outcomes of statistical inference (for both this algorithm
    and the NBS method) can vary markedly for even small changes to enhancement
    parameters. Therefore the specificity of results obtained using either of
    these methods should be interpreted with caution.
    
    In some software packages, a column of ones is automatically added to the
    GLM design matrix; the purpose of this column is to estimate the "global
    intercept", which is the predicted value of the observed variable if all
    explanatory variables were to be zero. However there are rare situations
    where including such a column would not be appropriate for a particular
    experimental design. Hence, in MRtrix3 statistical inference commands, it is
    up to the user to determine whether or not this column of ones should be
    included in their design matrix, and add it explicitly if necessary. The
    contrast matrix must also reflect the presence of this additional column.
    
    References:
    
    * If using the NBS algorithm:
    Zalesky, A.; Fornito, A. & Bullmore, E. T. Network-based statistic:
    Identifying differences in brain networks.
    NeuroImage, 2010, 53, 1197-1207
    
    * If using the TFNBS algorithm:
    Baggio, H.C.; Abos, A.; Segura, B.; Campabadal, A.; Garcia-Diaz, A.; Uribe,
    C.; Compta, Y.; Marti, M.J.; Valldeoriola, F.; Junque, C. Statistical
    inference in brain graphs using threshold-free network-based statistics.HBM,
    2018, 39, 2289-2302
    
    * If using the -nonstationary option:
    Salimi-Khorshidi, G.; Smith, S.M. & Nichols, T.E. Adjusting the effect of
    nonstationarity in cluster-based and TFCE inference.
    Neuroimage, 2011, 54(3), 2006-19.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        input_: a text file listing the file names of the input connectomes.
        algorithm: the algorithm to use in network-based\
            clustering/enhancement. Options are: nbs, tfnbs, none.
        design: the design matrix.
        contrast: the contrast matrix.
        output: the filename prefix for all output.
        notest: don't perform statistical inference; only output population\
            statistics (effect size, stdev etc).
        errors: specify nature of errors for shuffling; options are:\
            ee,ise,both (default: ee).
        exchange_within: specify blocks of observations within each of which\
            data may undergo restricted exchange.
        exchange_whole: specify blocks of observations that may be exchanged\
            with one another (for independent and symmetric errors, sign-flipping\
            will occur block-wise).
        strong: use strong familywise error control across multiple hypotheses.
        nshuffles: the number of shuffles (default: 5000).
        permutations: manually define the permutations (relabelling). The input\
            should be a text file defining a m x n matrix, where each relabelling\
            is defined as a column vector of size m, and the number of columns, n,\
            defines the number of permutations. Can be generated with the\
            palm_quickperms function in PALM\
            (http://fsl.fmrib.ox.ac.uk/fsl/fslwiki/PALM). Overrides the -nshuffles\
            option.
        nonstationarity: perform non-stationarity correction.
        skew_nonstationarity: specify the skew parameter for empirical\
            statistic calculation (default for this command is 1).
        nshuffles_nonstationarity: the number of shuffles to use when\
            precomputing the empirical statistic image for non-stationarity\
            correction (default: 5000).
        permutations_nonstationarity: manually define the permutations\
            (relabelling) for computing the emprical statistics for\
            non-stationarity correction. The input should be a text file defining a\
            m x n matrix, where each relabelling is defined as a column vector of\
            size m, and the number of columns, n, defines the number of\
            permutations. Can be generated with the palm_quickperms function in\
            PALM (http://fsl.fmrib.ox.ac.uk/fsl/fslwiki/PALM) Overrides the\
            -nshuffles_nonstationarity option.
        tfce_dh: the height increment used in the tfce integration (default:\
            0.1).
        tfce_e: tfce extent exponent (default: 0.4).
        tfce_h: tfce height exponent (default: 3).
        variance: define variance groups for the G-statistic; measurements for\
            which the expected variance is equivalent should contain the same index.
        ftests: perform F-tests; input text file should contain, for each\
            F-test, a row containing ones and zeros, where ones indicate the rows\
            of the contrast matrix to be included in the F-test.
        fonly: only assess F-tests; do not perform statistical inference on\
            entries in the contrast matrix.
        column: add a column to the design matrix corresponding to subject\
            edge-wise values (note that the contrast matrix must include an\
            additional column for each use of this option); the text file provided\
            via this option should contain a file name for each subject.
        threshold: the t-statistic value to use in threshold-based clustering\
            algorithms.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ConnectomestatsOutputs`).
    """
    params = connectomestats_params(
        notest=notest,
        errors=errors,
        exchange_within=exchange_within,
        exchange_whole=exchange_whole,
        strong=strong,
        nshuffles=nshuffles,
        permutations=permutations,
        nonstationarity=nonstationarity,
        skew_nonstationarity=skew_nonstationarity,
        nshuffles_nonstationarity=nshuffles_nonstationarity,
        permutations_nonstationarity=permutations_nonstationarity,
        tfce_dh=tfce_dh,
        tfce_e=tfce_e,
        tfce_h=tfce_h,
        variance=variance,
        ftests=ftests,
        fonly=fonly,
        column=column,
        threshold=threshold,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        input_=input_,
        algorithm=algorithm,
        design=design,
        contrast=contrast,
        output=output,
    )
    return connectomestats_execute(params, runner)


__all__ = [
    "CONNECTOMESTATS_METADATA",
    "ConnectomestatsColumnParamsDict",
    "ConnectomestatsColumnParamsDictTagged",
    "ConnectomestatsConfigParamsDict",
    "ConnectomestatsConfigParamsDictTagged",
    "ConnectomestatsOutputs",
    "ConnectomestatsParamsDict",
    "ConnectomestatsParamsDictTagged",
    "connectomestats",
    "connectomestats_column",
    "connectomestats_config",
    "connectomestats_execute",
    "connectomestats_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

WARPCONVERT_METADATA = Metadata(
    id="7468f4768b6e008e092ad5508a8d2b4875bab30a.boutiques",
    name="warpconvert",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_WarpconvertConfigParamsDictNoTag = typing.TypedDict('_WarpconvertConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
WarpconvertConfigParamsDictTagged = typing.TypedDict('WarpconvertConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
WarpconvertConfigParamsDict = _WarpconvertConfigParamsDictNoTag | WarpconvertConfigParamsDictTagged


_WarpconvertParamsDictNoTag = typing.TypedDict('_WarpconvertParamsDictNoTag', {
    "template": typing.NotRequired[InputPathType | None],
    "midway_space": bool,
    "from": typing.NotRequired[int | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[WarpconvertConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "in": InputPathType,
    "type": str,
    "out": str,
})
WarpconvertParamsDictTagged = typing.TypedDict('WarpconvertParamsDictTagged', {
    "@type": typing.Literal["mrtrix/warpconvert"],
    "template": typing.NotRequired[InputPathType | None],
    "midway_space": bool,
    "from": typing.NotRequired[int | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[WarpconvertConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "in": InputPathType,
    "type": str,
    "out": str,
})
WarpconvertParamsDict = _WarpconvertParamsDictNoTag | WarpconvertParamsDictTagged


def warpconvert_config(
    key: str,
    value: str,
) -> WarpconvertConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def warpconvert_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `WarpconvertConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def warpconvert_config_cargs(
    params: WarpconvertConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class WarpconvertOutputs(typing.NamedTuple):
    """
    Output object returned when calling `WarpconvertParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    out: OutputPathType
    """the output warp image."""


def warpconvert_params(
    in_: InputPathType,
    type_: str,
    out: str,
    template: InputPathType | None = None,
    midway_space: bool = False,
    from_: int | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[WarpconvertConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> WarpconvertParamsDictTagged:
    """
    Build parameters.
    
    Args:
        in_: the input warp image.
        type_: the conversion type required. Valid choices are:\
            deformation2displacement, displacement2deformation,\
            warpfull2deformation, warpfull2displacement.
        out: the output warp image.
        template: define a template image when converting a warpfull file\
            (which is defined on a grid in the midway space between image 1 & 2).\
            For example to generate the deformation field that maps image1 to\
            image2, then supply image2 as the template image.
        midway_space: to be used only with warpfull2deformation and\
            warpfull2displacement conversion types. The output will only contain\
            the non-linear warp to map an input image to the midway space (defined\
            by the warpfull grid). If a linear transform exists in the warpfull\
            file header then it will be composed and included in the output.
        from_: to be used only with warpfull2deformation and\
            warpfull2displacement conversion types. Used to define the direction of\
            the desired output field.Use -from 1 to obtain the image1->image2 field\
            and from 2 for image2->image1. Can be used in combination with the\
            -midway_space option to produce a field that only maps to midway space.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/warpconvert",
        "midway_space": midway_space,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "in": in_,
        "type": type_,
        "out": out,
    }
    if template is not None:
        params["template"] = template
    if from_ is not None:
        params["from"] = from_
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def warpconvert_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `WarpconvertParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("template", None) is not None:
        if not isinstance(params["template"], (pathlib.Path, str)):
            raise StyxValidationError(f'`template` has the wrong type: Received `{type(params.get("template", None))}` expected `InputPathType | None`')
    if params.get("midway_space", False) is None:
        raise StyxValidationError("`midway_space` must not be None")
    if not isinstance(params["midway_space"], bool):
        raise StyxValidationError(f'`midway_space` has the wrong type: Received `{type(params.get("midway_space", False))}` expected `bool`')
    if params.get("from", None) is not None:
        if not isinstance(params["from"], int):
            raise StyxValidationError(f'`from` has the wrong type: Received `{type(params.get("from", None))}` expected `int | None`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[WarpconvertConfigParamsDict] | None`')
        for e in params["config"]:
            warpconvert_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("in", None) is None:
        raise StyxValidationError("`in` must not be None")
    if not isinstance(params["in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`in` has the wrong type: Received `{type(params.get("in", None))}` expected `InputPathType`')
    if params.get("type", None) is None:
        raise StyxValidationError("`type` must not be None")
    if not isinstance(params["type"], str):
        raise StyxValidationError(f'`type` has the wrong type: Received `{type(params.get("type", None))}` expected `str`')
    if params.get("out", None) is None:
        raise StyxValidationError("`out` must not be None")
    if not isinstance(params["out"], str):
        raise StyxValidationError(f'`out` has the wrong type: Received `{type(params.get("out", None))}` expected `str`')


def warpconvert_cargs(
    params: WarpconvertParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("warpconvert")
    if params.get("template", None) is not None:
        cargs.extend([
            "-template",
            execution.input_file(params.get("template", None))
        ])
    if params.get("midway_space", False):
        cargs.append("-midway_space")
    if params.get("from", None) is not None:
        cargs.extend([
            "-from",
            str(params.get("from", None))
        ])
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [warpconvert_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("in", None)))
    cargs.append(params.get("type", None))
    cargs.append(params.get("out", None))
    return cargs


def warpconvert_outputs(
    params: WarpconvertParamsDict,
    execution: Execution,
) -> WarpconvertOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = WarpconvertOutputs(
        root=execution.output_file("."),
        out=execution.output_file(params.get("out", None)),
    )
    return ret


def warpconvert_execute(
    params: WarpconvertParamsDict,
    runner: Runner | None = None,
) -> WarpconvertOutputs:
    """
    warpconvert
    
    Convert between different representations of a non-linear warp.
    
    A deformation field is defined as an image where each voxel defines the
    corresponding position in the other image (in scanner space coordinates). A
    displacement field stores the displacements (in mm) to the other image from
    the each voxel's position (in scanner space). The warpfull file is the 5D
    format output from mrregister -nl_warp_full, which contains linear
    transforms, warps and their inverses that map each image to a midway space.
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `WarpconvertOutputs`).
    """
    warpconvert_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(WARPCONVERT_METADATA)
    params = execution.params(params)
    cargs = warpconvert_cargs(params, execution)
    ret = warpconvert_outputs(params, execution)
    execution.run(cargs)
    return ret


def warpconvert(
    in_: InputPathType,
    type_: str,
    out: str,
    template: InputPathType | None = None,
    midway_space: bool = False,
    from_: int | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[WarpconvertConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> WarpconvertOutputs:
    """
    warpconvert
    
    Convert between different representations of a non-linear warp.
    
    A deformation field is defined as an image where each voxel defines the
    corresponding position in the other image (in scanner space coordinates). A
    displacement field stores the displacements (in mm) to the other image from
    the each voxel's position (in scanner space). The warpfull file is the 5D
    format output from mrregister -nl_warp_full, which contains linear
    transforms, warps and their inverses that map each image to a midway space.
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        in_: the input warp image.
        type_: the conversion type required. Valid choices are:\
            deformation2displacement, displacement2deformation,\
            warpfull2deformation, warpfull2displacement.
        out: the output warp image.
        template: define a template image when converting a warpfull file\
            (which is defined on a grid in the midway space between image 1 & 2).\
            For example to generate the deformation field that maps image1 to\
            image2, then supply image2 as the template image.
        midway_space: to be used only with warpfull2deformation and\
            warpfull2displacement conversion types. The output will only contain\
            the non-linear warp to map an input image to the midway space (defined\
            by the warpfull grid). If a linear transform exists in the warpfull\
            file header then it will be composed and included in the output.
        from_: to be used only with warpfull2deformation and\
            warpfull2displacement conversion types. Used to define the direction of\
            the desired output field.Use -from 1 to obtain the image1->image2 field\
            and from 2 for image2->image1. Can be used in combination with the\
            -midway_space option to produce a field that only maps to midway space.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `WarpconvertOutputs`).
    """
    params = warpconvert_params(
        template=template,
        midway_space=midway_space,
        from_=from_,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        in_=in_,
        type_=type_,
        out=out,
    )
    return warpconvert_execute(params, runner)


__all__ = [
    "WARPCONVERT_METADATA",
    "WarpconvertConfigParamsDict",
    "WarpconvertConfigParamsDictTagged",
    "WarpconvertOutputs",
    "WarpconvertParamsDict",
    "WarpconvertParamsDictTagged",
    "warpconvert",
    "warpconvert_config",
    "warpconvert_execute",
    "warpconvert_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

VOXEL2MESH_METADATA = Metadata(
    id="85ff2c9f04efcbe2d5484d2e2ccd18c1577521ed.boutiques",
    name="voxel2mesh",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_Voxel2meshConfigParamsDictNoTag = typing.TypedDict('_Voxel2meshConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
Voxel2meshConfigParamsDictTagged = typing.TypedDict('Voxel2meshConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
Voxel2meshConfigParamsDict = _Voxel2meshConfigParamsDictNoTag | Voxel2meshConfigParamsDictTagged


_Voxel2meshParamsDictNoTag = typing.TypedDict('_Voxel2meshParamsDictNoTag', {
    "blocky": bool,
    "threshold": typing.NotRequired[float | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[Voxel2meshConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "input": InputPathType,
    "output": str,
})
Voxel2meshParamsDictTagged = typing.TypedDict('Voxel2meshParamsDictTagged', {
    "@type": typing.Literal["mrtrix/voxel2mesh"],
    "blocky": bool,
    "threshold": typing.NotRequired[float | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[Voxel2meshConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "input": InputPathType,
    "output": str,
})
Voxel2meshParamsDict = _Voxel2meshParamsDictNoTag | Voxel2meshParamsDictTagged


def voxel2mesh_config(
    key: str,
    value: str,
) -> Voxel2meshConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def voxel2mesh_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Voxel2meshConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def voxel2mesh_config_cargs(
    params: Voxel2meshConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class Voxel2meshOutputs(typing.NamedTuple):
    """
    Output object returned when calling `Voxel2meshParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output: OutputPathType
    """the output mesh file."""


def voxel2mesh_params(
    input_: InputPathType,
    output: str,
    blocky: bool = False,
    threshold: float | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[Voxel2meshConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> Voxel2meshParamsDictTagged:
    """
    Build parameters.
    
    Args:
        input_: the input image.
        output: the output mesh file.
        blocky: generate a 'blocky' mesh that precisely represents the voxel\
            edges.
        threshold: manually set the intensity threshold for the Marching Cubes\
            algorithm.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/voxel2mesh",
        "blocky": blocky,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "input": input_,
        "output": output,
    }
    if threshold is not None:
        params["threshold"] = threshold
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def voxel2mesh_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Voxel2meshParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("blocky", False) is None:
        raise StyxValidationError("`blocky` must not be None")
    if not isinstance(params["blocky"], bool):
        raise StyxValidationError(f'`blocky` has the wrong type: Received `{type(params.get("blocky", False))}` expected `bool`')
    if params.get("threshold", None) is not None:
        if not isinstance(params["threshold"], (float, int)):
            raise StyxValidationError(f'`threshold` has the wrong type: Received `{type(params.get("threshold", None))}` expected `float | None`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[Voxel2meshConfigParamsDict] | None`')
        for e in params["config"]:
            voxel2mesh_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("input", None) is None:
        raise StyxValidationError("`input` must not be None")
    if not isinstance(params["input"], (pathlib.Path, str)):
        raise StyxValidationError(f'`input` has the wrong type: Received `{type(params.get("input", None))}` expected `InputPathType`')
    if params.get("output", None) is None:
        raise StyxValidationError("`output` must not be None")
    if not isinstance(params["output"], str):
        raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str`')


def voxel2mesh_cargs(
    params: Voxel2meshParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("voxel2mesh")
    if params.get("blocky", False):
        cargs.append("-blocky")
    if params.get("threshold", None) is not None:
        cargs.extend([
            "-threshold",
            str(params.get("threshold", None))
        ])
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [voxel2mesh_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("input", None)))
    cargs.append(params.get("output", None))
    return cargs


def voxel2mesh_outputs(
    params: Voxel2meshParamsDict,
    execution: Execution,
) -> Voxel2meshOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = Voxel2meshOutputs(
        root=execution.output_file("."),
        output=execution.output_file(params.get("output", None)),
    )
    return ret


def voxel2mesh_execute(
    params: Voxel2meshParamsDict,
    runner: Runner | None = None,
) -> Voxel2meshOutputs:
    """
    voxel2mesh
    
    Generate a surface mesh representation from a voxel image.
    
    This command utilises the Marching Cubes algorithm to generate a polygonal
    surface that represents the isocontour(s) of the input image at a particular
    intensity. By default, an appropriate threshold will be determined
    automatically from the input image, however the intensity value of the
    isocontour(s) can instead be set manually using the -threhsold option.
    
    If the -blocky option is used, then the Marching Cubes algorithm will not be
    used. Instead, the input image will be interpreted as a binary mask image,
    and polygonal surfaces will be generated at the outer faces of the voxel
    clusters within the mask.
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Voxel2meshOutputs`).
    """
    voxel2mesh_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOXEL2MESH_METADATA)
    params = execution.params(params)
    cargs = voxel2mesh_cargs(params, execution)
    ret = voxel2mesh_outputs(params, execution)
    execution.run(cargs)
    return ret


def voxel2mesh(
    input_: InputPathType,
    output: str,
    blocky: bool = False,
    threshold: float | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[Voxel2meshConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> Voxel2meshOutputs:
    """
    voxel2mesh
    
    Generate a surface mesh representation from a voxel image.
    
    This command utilises the Marching Cubes algorithm to generate a polygonal
    surface that represents the isocontour(s) of the input image at a particular
    intensity. By default, an appropriate threshold will be determined
    automatically from the input image, however the intensity value of the
    isocontour(s) can instead be set manually using the -threhsold option.
    
    If the -blocky option is used, then the Marching Cubes algorithm will not be
    used. Instead, the input image will be interpreted as a binary mask image,
    and polygonal surfaces will be generated at the outer faces of the voxel
    clusters within the mask.
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        input_: the input image.
        output: the output mesh file.
        blocky: generate a 'blocky' mesh that precisely represents the voxel\
            edges.
        threshold: manually set the intensity threshold for the Marching Cubes\
            algorithm.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Voxel2meshOutputs`).
    """
    params = voxel2mesh_params(
        blocky=blocky,
        threshold=threshold,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        input_=input_,
        output=output,
    )
    return voxel2mesh_execute(params, runner)


__all__ = [
    "VOXEL2MESH_METADATA",
    "Voxel2meshConfigParamsDict",
    "Voxel2meshConfigParamsDictTagged",
    "Voxel2meshOutputs",
    "Voxel2meshParamsDict",
    "Voxel2meshParamsDictTagged",
    "voxel2mesh",
    "voxel2mesh_config",
    "voxel2mesh_execute",
    "voxel2mesh_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CONNECTOME2TCK_METADATA = Metadata(
    id="ae2ef98104119bc0342c1596e2bea3d807ec75ef.boutiques",
    name="connectome2tck",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_Connectome2tckConfigParamsDictNoTag = typing.TypedDict('_Connectome2tckConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
Connectome2tckConfigParamsDictTagged = typing.TypedDict('Connectome2tckConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
Connectome2tckConfigParamsDict = _Connectome2tckConfigParamsDictNoTag | Connectome2tckConfigParamsDictTagged


_Connectome2tckParamsDictNoTag = typing.TypedDict('_Connectome2tckParamsDictNoTag', {
    "nodes": typing.NotRequired[list[int] | None],
    "exclusive": bool,
    "files": typing.NotRequired[str | None],
    "exemplars": typing.NotRequired[InputPathType | None],
    "keep_unassigned": bool,
    "keep_self": bool,
    "tck_weights_in": typing.NotRequired[InputPathType | None],
    "prefix_tck_weights_out": typing.NotRequired[str | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[Connectome2tckConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "tracks_in": InputPathType,
    "assignments_in": InputPathType,
    "prefix_out": str,
})
Connectome2tckParamsDictTagged = typing.TypedDict('Connectome2tckParamsDictTagged', {
    "@type": typing.Literal["mrtrix/connectome2tck"],
    "nodes": typing.NotRequired[list[int] | None],
    "exclusive": bool,
    "files": typing.NotRequired[str | None],
    "exemplars": typing.NotRequired[InputPathType | None],
    "keep_unassigned": bool,
    "keep_self": bool,
    "tck_weights_in": typing.NotRequired[InputPathType | None],
    "prefix_tck_weights_out": typing.NotRequired[str | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[Connectome2tckConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "tracks_in": InputPathType,
    "assignments_in": InputPathType,
    "prefix_out": str,
})
Connectome2tckParamsDict = _Connectome2tckParamsDictNoTag | Connectome2tckParamsDictTagged


def connectome2tck_config(
    key: str,
    value: str,
) -> Connectome2tckConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def connectome2tck_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Connectome2tckConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def connectome2tck_config_cargs(
    params: Connectome2tckConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class Connectome2tckOutputs(typing.NamedTuple):
    """
    Output object returned when calling `Connectome2tckParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def connectome2tck_params(
    tracks_in: InputPathType,
    assignments_in: InputPathType,
    prefix_out: str,
    nodes: list[int] | None = None,
    exclusive: bool = False,
    files: str | None = None,
    exemplars: InputPathType | None = None,
    keep_unassigned: bool = False,
    keep_self: bool = False,
    tck_weights_in: InputPathType | None = None,
    prefix_tck_weights_out: str | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[Connectome2tckConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> Connectome2tckParamsDictTagged:
    """
    Build parameters.
    
    Args:
        tracks_in: the input track file.
        assignments_in: input text file containing the node assignments for\
            each streamline.
        prefix_out: the output file / prefix.
        nodes: only select tracks that involve a set of nodes of interest\
            (provide as a comma-separated list of integers).
        exclusive: only select tracks that exclusively connect nodes from\
            within the list of nodes of interest.
        files: select how the resulting streamlines will be grouped in output\
            files. Options are: per_edge, per_node, single (default: per_edge).
        exemplars: generate a mean connection exemplar per edge, rather than\
            keeping all streamlines (the parcellation node image must be provided\
            in order to constrain the exemplar endpoints).
        keep_unassigned: by default, the program discards those streamlines\
            that are not successfully assigned to a node. Set this option to\
            generate corresponding outputs containing these streamlines (labelled\
            as node index 0).
        keep_self: by default, the program will not output streamlines that\
            connect to the same node at both ends. Set this option to instead keep\
            these self-connections.
        tck_weights_in: specify a text scalar file containing the streamline\
            weights.
        prefix_tck_weights_out: provide a prefix for outputting a text file\
            corresponding to each output file, each containing only the streamline\
            weights relevant for that track file.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/connectome2tck",
        "exclusive": exclusive,
        "keep_unassigned": keep_unassigned,
        "keep_self": keep_self,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "tracks_in": tracks_in,
        "assignments_in": assignments_in,
        "prefix_out": prefix_out,
    }
    if nodes is not None:
        params["nodes"] = nodes
    if files is not None:
        params["files"] = files
    if exemplars is not None:
        params["exemplars"] = exemplars
    if tck_weights_in is not None:
        params["tck_weights_in"] = tck_weights_in
    if prefix_tck_weights_out is not None:
        params["prefix_tck_weights_out"] = prefix_tck_weights_out
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def connectome2tck_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Connectome2tckParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("nodes", None) is not None:
        if not isinstance(params["nodes"], list):
            raise StyxValidationError(f'`nodes` has the wrong type: Received `{type(params.get("nodes", None))}` expected `list[int] | None`')
        for e in params["nodes"]:
            if not isinstance(e, int):
                raise StyxValidationError(f'`nodes` has the wrong type: Received `{type(params.get("nodes", None))}` expected `list[int] | None`')
    if params.get("exclusive", False) is None:
        raise StyxValidationError("`exclusive` must not be None")
    if not isinstance(params["exclusive"], bool):
        raise StyxValidationError(f'`exclusive` has the wrong type: Received `{type(params.get("exclusive", False))}` expected `bool`')
    if params.get("files", None) is not None:
        if not isinstance(params["files"], str):
            raise StyxValidationError(f'`files` has the wrong type: Received `{type(params.get("files", None))}` expected `str | None`')
    if params.get("exemplars", None) is not None:
        if not isinstance(params["exemplars"], (pathlib.Path, str)):
            raise StyxValidationError(f'`exemplars` has the wrong type: Received `{type(params.get("exemplars", None))}` expected `InputPathType | None`')
    if params.get("keep_unassigned", False) is None:
        raise StyxValidationError("`keep_unassigned` must not be None")
    if not isinstance(params["keep_unassigned"], bool):
        raise StyxValidationError(f'`keep_unassigned` has the wrong type: Received `{type(params.get("keep_unassigned", False))}` expected `bool`')
    if params.get("keep_self", False) is None:
        raise StyxValidationError("`keep_self` must not be None")
    if not isinstance(params["keep_self"], bool):
        raise StyxValidationError(f'`keep_self` has the wrong type: Received `{type(params.get("keep_self", False))}` expected `bool`')
    if params.get("tck_weights_in", None) is not None:
        if not isinstance(params["tck_weights_in"], (pathlib.Path, str)):
            raise StyxValidationError(f'`tck_weights_in` has the wrong type: Received `{type(params.get("tck_weights_in", None))}` expected `InputPathType | None`')
    if params.get("prefix_tck_weights_out", None) is not None:
        if not isinstance(params["prefix_tck_weights_out"], str):
            raise StyxValidationError(f'`prefix_tck_weights_out` has the wrong type: Received `{type(params.get("prefix_tck_weights_out", None))}` expected `str | None`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[Connectome2tckConfigParamsDict] | None`')
        for e in params["config"]:
            connectome2tck_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("tracks_in", None) is None:
        raise StyxValidationError("`tracks_in` must not be None")
    if not isinstance(params["tracks_in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`tracks_in` has the wrong type: Received `{type(params.get("tracks_in", None))}` expected `InputPathType`')
    if params.get("assignments_in", None) is None:
        raise StyxValidationError("`assignments_in` must not be None")
    if not isinstance(params["assignments_in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`assignments_in` has the wrong type: Received `{type(params.get("assignments_in", None))}` expected `InputPathType`')
    if params.get("prefix_out", None) is None:
        raise StyxValidationError("`prefix_out` must not be None")
    if not isinstance(params["prefix_out"], str):
        raise StyxValidationError(f'`prefix_out` has the wrong type: Received `{type(params.get("prefix_out", None))}` expected `str`')


def connectome2tck_cargs(
    params: Connectome2tckParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("connectome2tck")
    if params.get("nodes", None) is not None:
        cargs.extend([
            "-nodes",
            ",".join(map(str, params.get("nodes", None)))
        ])
    if params.get("exclusive", False):
        cargs.append("-exclusive")
    if params.get("files", None) is not None:
        cargs.extend([
            "-files",
            params.get("files", None)
        ])
    if params.get("exemplars", None) is not None:
        cargs.extend([
            "-exemplars",
            execution.input_file(params.get("exemplars", None))
        ])
    if params.get("keep_unassigned", False):
        cargs.append("-keep_unassigned")
    if params.get("keep_self", False):
        cargs.append("-keep_self")
    if params.get("tck_weights_in", None) is not None:
        cargs.extend([
            "-tck_weights_in",
            execution.input_file(params.get("tck_weights_in", None))
        ])
    if params.get("prefix_tck_weights_out", None) is not None:
        cargs.extend([
            "-prefix_tck_weights_out",
            params.get("prefix_tck_weights_out", None)
        ])
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [connectome2tck_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("tracks_in", None)))
    cargs.append(execution.input_file(params.get("assignments_in", None)))
    cargs.append(params.get("prefix_out", None))
    return cargs


def connectome2tck_outputs(
    params: Connectome2tckParamsDict,
    execution: Execution,
) -> Connectome2tckOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = Connectome2tckOutputs(
        root=execution.output_file("."),
    )
    return ret


def connectome2tck_execute(
    params: Connectome2tckParamsDict,
    runner: Runner | None = None,
) -> Connectome2tckOutputs:
    """
    connectome2tck
    
    Extract streamlines from a tractogram based on their assignment to
    parcellated nodes.
    
    The compulsory input file "assignments_in" should contain a text file where
    there is one row for each streamline, and each row contains a list of
    numbers corresponding to the parcels to which that streamline was assigned
    (most typically there will be two entries per streamline, one for each
    endpoint; but this is not strictly a requirement). This file will most
    typically be generated using the tck2connectome command with the
    -out_assignments option.
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Connectome2tckOutputs`).
    """
    connectome2tck_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CONNECTOME2TCK_METADATA)
    params = execution.params(params)
    cargs = connectome2tck_cargs(params, execution)
    ret = connectome2tck_outputs(params, execution)
    execution.run(cargs)
    return ret


def connectome2tck(
    tracks_in: InputPathType,
    assignments_in: InputPathType,
    prefix_out: str,
    nodes: list[int] | None = None,
    exclusive: bool = False,
    files: str | None = None,
    exemplars: InputPathType | None = None,
    keep_unassigned: bool = False,
    keep_self: bool = False,
    tck_weights_in: InputPathType | None = None,
    prefix_tck_weights_out: str | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[Connectome2tckConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> Connectome2tckOutputs:
    """
    connectome2tck
    
    Extract streamlines from a tractogram based on their assignment to
    parcellated nodes.
    
    The compulsory input file "assignments_in" should contain a text file where
    there is one row for each streamline, and each row contains a list of
    numbers corresponding to the parcels to which that streamline was assigned
    (most typically there will be two entries per streamline, one for each
    endpoint; but this is not strictly a requirement). This file will most
    typically be generated using the tck2connectome command with the
    -out_assignments option.
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        tracks_in: the input track file.
        assignments_in: input text file containing the node assignments for\
            each streamline.
        prefix_out: the output file / prefix.
        nodes: only select tracks that involve a set of nodes of interest\
            (provide as a comma-separated list of integers).
        exclusive: only select tracks that exclusively connect nodes from\
            within the list of nodes of interest.
        files: select how the resulting streamlines will be grouped in output\
            files. Options are: per_edge, per_node, single (default: per_edge).
        exemplars: generate a mean connection exemplar per edge, rather than\
            keeping all streamlines (the parcellation node image must be provided\
            in order to constrain the exemplar endpoints).
        keep_unassigned: by default, the program discards those streamlines\
            that are not successfully assigned to a node. Set this option to\
            generate corresponding outputs containing these streamlines (labelled\
            as node index 0).
        keep_self: by default, the program will not output streamlines that\
            connect to the same node at both ends. Set this option to instead keep\
            these self-connections.
        tck_weights_in: specify a text scalar file containing the streamline\
            weights.
        prefix_tck_weights_out: provide a prefix for outputting a text file\
            corresponding to each output file, each containing only the streamline\
            weights relevant for that track file.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Connectome2tckOutputs`).
    """
    params = connectome2tck_params(
        nodes=nodes,
        exclusive=exclusive,
        files=files,
        exemplars=exemplars,
        keep_unassigned=keep_unassigned,
        keep_self=keep_self,
        tck_weights_in=tck_weights_in,
        prefix_tck_weights_out=prefix_tck_weights_out,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        tracks_in=tracks_in,
        assignments_in=assignments_in,
        prefix_out=prefix_out,
    )
    return connectome2tck_execute(params, runner)


__all__ = [
    "CONNECTOME2TCK_METADATA",
    "Connectome2tckConfigParamsDict",
    "Connectome2tckConfigParamsDictTagged",
    "Connectome2tckOutputs",
    "Connectome2tckParamsDict",
    "Connectome2tckParamsDictTagged",
    "connectome2tck",
    "connectome2tck_config",
    "connectome2tck_execute",
    "connectome2tck_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

DWI2ADC_METADATA = Metadata(
    id="8ba8efdde3cc0a5ed2e0c5a5f36fc02ee6f162a1.boutiques",
    name="dwi2adc",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_Dwi2adcFslgradParamsDictNoTag = typing.TypedDict('_Dwi2adcFslgradParamsDictNoTag', {
    "bvecs": InputPathType,
    "bvals": InputPathType,
})
Dwi2adcFslgradParamsDictTagged = typing.TypedDict('Dwi2adcFslgradParamsDictTagged', {
    "@type": typing.Literal["fslgrad"],
    "bvecs": InputPathType,
    "bvals": InputPathType,
})
Dwi2adcFslgradParamsDict = _Dwi2adcFslgradParamsDictNoTag | Dwi2adcFslgradParamsDictTagged


_Dwi2adcConfigParamsDictNoTag = typing.TypedDict('_Dwi2adcConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
Dwi2adcConfigParamsDictTagged = typing.TypedDict('Dwi2adcConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
Dwi2adcConfigParamsDict = _Dwi2adcConfigParamsDictNoTag | Dwi2adcConfigParamsDictTagged


_Dwi2adcParamsDictNoTag = typing.TypedDict('_Dwi2adcParamsDictNoTag', {
    "grad": typing.NotRequired[InputPathType | None],
    "fslgrad": typing.NotRequired[Dwi2adcFslgradParamsDict | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[Dwi2adcConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "input": InputPathType,
    "output": str,
})
Dwi2adcParamsDictTagged = typing.TypedDict('Dwi2adcParamsDictTagged', {
    "@type": typing.Literal["mrtrix/dwi2adc"],
    "grad": typing.NotRequired[InputPathType | None],
    "fslgrad": typing.NotRequired[Dwi2adcFslgradParamsDict | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[Dwi2adcConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "input": InputPathType,
    "output": str,
})
Dwi2adcParamsDict = _Dwi2adcParamsDictNoTag | Dwi2adcParamsDictTagged


def dwi2adc_fslgrad(
    bvecs: InputPathType,
    bvals: InputPathType,
) -> Dwi2adcFslgradParamsDictTagged:
    """
    Build parameters.
    
    Args:
        bvecs: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
        bvals: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "fslgrad",
        "bvecs": bvecs,
        "bvals": bvals,
    }
    return params


def dwi2adc_fslgrad_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Dwi2adcFslgradParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("bvecs", None) is None:
        raise StyxValidationError("`bvecs` must not be None")
    if not isinstance(params["bvecs"], (pathlib.Path, str)):
        raise StyxValidationError(f'`bvecs` has the wrong type: Received `{type(params.get("bvecs", None))}` expected `InputPathType`')
    if params.get("bvals", None) is None:
        raise StyxValidationError("`bvals` must not be None")
    if not isinstance(params["bvals"], (pathlib.Path, str)):
        raise StyxValidationError(f'`bvals` has the wrong type: Received `{type(params.get("bvals", None))}` expected `InputPathType`')


def dwi2adc_fslgrad_cargs(
    params: Dwi2adcFslgradParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-fslgrad")
    cargs.append(execution.input_file(params.get("bvecs", None)))
    cargs.append(execution.input_file(params.get("bvals", None)))
    return cargs


def dwi2adc_config(
    key: str,
    value: str,
) -> Dwi2adcConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def dwi2adc_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Dwi2adcConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def dwi2adc_config_cargs(
    params: Dwi2adcConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class Dwi2adcOutputs(typing.NamedTuple):
    """
    Output object returned when calling `Dwi2adcParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output: OutputPathType
    """the output image."""


def dwi2adc_params(
    input_: InputPathType,
    output: str,
    grad: InputPathType | None = None,
    fslgrad: Dwi2adcFslgradParamsDict | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[Dwi2adcConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> Dwi2adcParamsDictTagged:
    """
    Build parameters.
    
    Args:
        input_: the input image.
        output: the output image.
        grad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in a text file. This should be supplied as a 4xN text file\
            with each line is in the format [ X Y Z b ], where [ X Y Z ] describe\
            the direction of the applied gradient, and b gives the b-value in units\
            of s/mm^2. If a diffusion gradient scheme is present in the input image\
            header, the data provided with this option will be instead used.
        fslgrad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/dwi2adc",
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "input": input_,
        "output": output,
    }
    if grad is not None:
        params["grad"] = grad
    if fslgrad is not None:
        params["fslgrad"] = fslgrad
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def dwi2adc_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Dwi2adcParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("grad", None) is not None:
        if not isinstance(params["grad"], (pathlib.Path, str)):
            raise StyxValidationError(f'`grad` has the wrong type: Received `{type(params.get("grad", None))}` expected `InputPathType | None`')
    if params.get("fslgrad", None) is not None:
        dwi2adc_fslgrad_validate(params["fslgrad"])
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[Dwi2adcConfigParamsDict] | None`')
        for e in params["config"]:
            dwi2adc_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("input", None) is None:
        raise StyxValidationError("`input` must not be None")
    if not isinstance(params["input"], (pathlib.Path, str)):
        raise StyxValidationError(f'`input` has the wrong type: Received `{type(params.get("input", None))}` expected `InputPathType`')
    if params.get("output", None) is None:
        raise StyxValidationError("`output` must not be None")
    if not isinstance(params["output"], str):
        raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str`')


def dwi2adc_cargs(
    params: Dwi2adcParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("dwi2adc")
    if params.get("grad", None) is not None:
        cargs.extend([
            "-grad",
            execution.input_file(params.get("grad", None))
        ])
    if params.get("fslgrad", None) is not None:
        cargs.extend(dwi2adc_fslgrad_cargs(params.get("fslgrad", None), execution))
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [dwi2adc_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("input", None)))
    cargs.append(params.get("output", None))
    return cargs


def dwi2adc_outputs(
    params: Dwi2adcParamsDict,
    execution: Execution,
) -> Dwi2adcOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = Dwi2adcOutputs(
        root=execution.output_file("."),
        output=execution.output_file(params.get("output", None)),
    )
    return ret


def dwi2adc_execute(
    params: Dwi2adcParamsDict,
    runner: Runner | None = None,
) -> Dwi2adcOutputs:
    """
    dwi2adc
    
    Convert mean dwi (trace-weighted) images to mean ADC maps.
    
    
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Dwi2adcOutputs`).
    """
    dwi2adc_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(DWI2ADC_METADATA)
    params = execution.params(params)
    cargs = dwi2adc_cargs(params, execution)
    ret = dwi2adc_outputs(params, execution)
    execution.run(cargs)
    return ret


def dwi2adc(
    input_: InputPathType,
    output: str,
    grad: InputPathType | None = None,
    fslgrad: Dwi2adcFslgradParamsDict | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[Dwi2adcConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> Dwi2adcOutputs:
    """
    dwi2adc
    
    Convert mean dwi (trace-weighted) images to mean ADC maps.
    
    
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        input_: the input image.
        output: the output image.
        grad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in a text file. This should be supplied as a 4xN text file\
            with each line is in the format [ X Y Z b ], where [ X Y Z ] describe\
            the direction of the applied gradient, and b gives the b-value in units\
            of s/mm^2. If a diffusion gradient scheme is present in the input image\
            header, the data provided with this option will be instead used.
        fslgrad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Dwi2adcOutputs`).
    """
    params = dwi2adc_params(
        grad=grad,
        fslgrad=fslgrad,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        input_=input_,
        output=output,
    )
    return dwi2adc_execute(params, runner)


__all__ = [
    "DWI2ADC_METADATA",
    "Dwi2adcConfigParamsDict",
    "Dwi2adcConfigParamsDictTagged",
    "Dwi2adcFslgradParamsDict",
    "Dwi2adcFslgradParamsDictTagged",
    "Dwi2adcOutputs",
    "Dwi2adcParamsDict",
    "Dwi2adcParamsDictTagged",
    "dwi2adc",
    "dwi2adc_config",
    "dwi2adc_execute",
    "dwi2adc_fslgrad",
    "dwi2adc_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

FIXEL2SH_METADATA = Metadata(
    id="8c1f4206ebf1611310a84b3b1b1527e0a6b154c6.boutiques",
    name="fixel2sh",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_Fixel2shConfigParamsDictNoTag = typing.TypedDict('_Fixel2shConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
Fixel2shConfigParamsDictTagged = typing.TypedDict('Fixel2shConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
Fixel2shConfigParamsDict = _Fixel2shConfigParamsDictNoTag | Fixel2shConfigParamsDictTagged


_Fixel2shParamsDictNoTag = typing.TypedDict('_Fixel2shParamsDictNoTag', {
    "lmax": typing.NotRequired[int | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[Fixel2shConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "fixel_in": InputPathType,
    "sh_out": str,
})
Fixel2shParamsDictTagged = typing.TypedDict('Fixel2shParamsDictTagged', {
    "@type": typing.Literal["mrtrix/fixel2sh"],
    "lmax": typing.NotRequired[int | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[Fixel2shConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "fixel_in": InputPathType,
    "sh_out": str,
})
Fixel2shParamsDict = _Fixel2shParamsDictNoTag | Fixel2shParamsDictTagged


def fixel2sh_config(
    key: str,
    value: str,
) -> Fixel2shConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def fixel2sh_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Fixel2shConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def fixel2sh_config_cargs(
    params: Fixel2shConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class Fixel2shOutputs(typing.NamedTuple):
    """
    Output object returned when calling `Fixel2shParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    sh_out: OutputPathType
    """the output sh image."""


def fixel2sh_params(
    fixel_in: InputPathType,
    sh_out: str,
    lmax: int | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[Fixel2shConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> Fixel2shParamsDictTagged:
    """
    Build parameters.
    
    Args:
        fixel_in: the input fixel data file.
        sh_out: the output sh image.
        lmax: set the maximum harmonic order for the output series (Default: 8).
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/fixel2sh",
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "fixel_in": fixel_in,
        "sh_out": sh_out,
    }
    if lmax is not None:
        params["lmax"] = lmax
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def fixel2sh_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Fixel2shParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("lmax", None) is not None:
        if not isinstance(params["lmax"], int):
            raise StyxValidationError(f'`lmax` has the wrong type: Received `{type(params.get("lmax", None))}` expected `int | None`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[Fixel2shConfigParamsDict] | None`')
        for e in params["config"]:
            fixel2sh_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("fixel_in", None) is None:
        raise StyxValidationError("`fixel_in` must not be None")
    if not isinstance(params["fixel_in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`fixel_in` has the wrong type: Received `{type(params.get("fixel_in", None))}` expected `InputPathType`')
    if params.get("sh_out", None) is None:
        raise StyxValidationError("`sh_out` must not be None")
    if not isinstance(params["sh_out"], str):
        raise StyxValidationError(f'`sh_out` has the wrong type: Received `{type(params.get("sh_out", None))}` expected `str`')


def fixel2sh_cargs(
    params: Fixel2shParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("fixel2sh")
    if params.get("lmax", None) is not None:
        cargs.extend([
            "-lmax",
            str(params.get("lmax", None))
        ])
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [fixel2sh_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("fixel_in", None)))
    cargs.append(params.get("sh_out", None))
    return cargs


def fixel2sh_outputs(
    params: Fixel2shParamsDict,
    execution: Execution,
) -> Fixel2shOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = Fixel2shOutputs(
        root=execution.output_file("."),
        sh_out=execution.output_file(params.get("sh_out", None)),
    )
    return ret


def fixel2sh_execute(
    params: Fixel2shParamsDict,
    runner: Runner | None = None,
) -> Fixel2shOutputs:
    """
    fixel2sh
    
    Convert a fixel-based sparse-data image into an spherical harmonic image.
    
    This command generates spherical harmonic data from fixels that can be
    visualised using the ODF tool in MRview. The output ODF lobes are scaled
    according to the values in the input fixel image.
    
    The spherical harmonic coefficients are stored according the conventions
    described the main documentation, which can be found at the following link:
    https://mrtrix.readthedocs.io/en/3.0.4/concepts/spherical_harmonics.html
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Fixel2shOutputs`).
    """
    fixel2sh_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(FIXEL2SH_METADATA)
    params = execution.params(params)
    cargs = fixel2sh_cargs(params, execution)
    ret = fixel2sh_outputs(params, execution)
    execution.run(cargs)
    return ret


def fixel2sh(
    fixel_in: InputPathType,
    sh_out: str,
    lmax: int | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[Fixel2shConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> Fixel2shOutputs:
    """
    fixel2sh
    
    Convert a fixel-based sparse-data image into an spherical harmonic image.
    
    This command generates spherical harmonic data from fixels that can be
    visualised using the ODF tool in MRview. The output ODF lobes are scaled
    according to the values in the input fixel image.
    
    The spherical harmonic coefficients are stored according the conventions
    described the main documentation, which can be found at the following link:
    https://mrtrix.readthedocs.io/en/3.0.4/concepts/spherical_harmonics.html
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        fixel_in: the input fixel data file.
        sh_out: the output sh image.
        lmax: set the maximum harmonic order for the output series (Default: 8).
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Fixel2shOutputs`).
    """
    params = fixel2sh_params(
        lmax=lmax,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        fixel_in=fixel_in,
        sh_out=sh_out,
    )
    return fixel2sh_execute(params, runner)


__all__ = [
    "FIXEL2SH_METADATA",
    "Fixel2shConfigParamsDict",
    "Fixel2shConfigParamsDictTagged",
    "Fixel2shOutputs",
    "Fixel2shParamsDict",
    "Fixel2shParamsDictTagged",
    "fixel2sh",
    "fixel2sh_config",
    "fixel2sh_execute",
    "fixel2sh_params",
]
